
# import libraries
library(readxl)
library(xlsx)
library(raster)
library(tidyverse)
library(zoo)
library(ggplot)
library(RColorBrewer)
library(sf)
library(viridis)
library(lubridate)      

###########################################################################################################################################################  
#1. extract P surplus from grid to NUTS level
###########################################################################################################################################################  

# Following script is an example to estimate P surplus at NUTS 0 level

# The shape file for Nomenclature of Territorial Units for Statistics (NUTS) units are obtained from European Commission, Eurostat (ESTAT), GISCO (https://ec.europa.eu/eurostat/web/gisco/geodata/reference-data/administrative-units-statistical-units/nuts)
############################## transform Coordinate Reference Systems (CRS) ############################## 
## transform Coordinate Reference Systems (CRS) of the NUTS shapefiles to a common CRS to make them align with other datasets
#  original projection of the NUTS shapefile     = "+proj=laea +lat_0=52 +lon_0=10 +x_0=4321000 +y_0=3210000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs"
#  transformed projection of the NUTS shapefile = "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"


Projected_CRS = "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"
# Import shape file at NUTS_0 level 
NUTS_0  <-  st_read("general/NUTS/ref-nuts-2021-01m.shp/NUTS_RG_01M_2021_3035_LEVL_0.shp/NUTS_RG_01M_2021_3035_LEVL_0.shp", stringsAsFactors=FALSE)
# transform projection
p_NUTS_0  <- st_transform(NUTS_0, CRS = Projected_CRS)


############################## Assign unique ID's to NUTS polygons ############################## 
# Derive information for the NUTS given them unique ID's
# Import file for target study domain
# The file is obtained from HYDE version 3.2.1 (directory "General files", file = "iso_cr_asc"). This file consist of total national iso numerical country codes, to get the target domain, we croped the file 
EU <- raster("general/Hyde/Eu_iso_cr_hyde.tif") 

# Import projected NUTS shapefile
NUTS_0 <- st_read(paste0("general/NUTS/ref-nuts-2021-01m.shp/Project_NUTS_", NUTS, ".shp"))
# Make a raster file of NUTS at target domain (European countries)
r_NUTS_0 <- rasterize(NUTS_0, EU)
# Convert the raster data into a dataframe
df <- r_NUTS_0 %>%
  as.data.frame(., xy = F) %>%
  as.data.frame(.)%>%
  group_by(NUTS_ID)%>%        # group the dataframe with the NUTS ID
  slice_head %>%              # obtain one row for every NUTS
  filter(NUTS_ID != "NA")     # filter out NA's
df$ID <- c(1:37)  # NUTS 0 has 37 units, thus we assigned unique ID's to those NUTS units

write.csv(df, "general/NUTS/NUTS_0_ID.csv") ## write NUTS ID's in csv format
# create a list for 16 N surplus estimates

methods_list <- c("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", 
                  "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", 
                  "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", 
                  "37", "38", "39", "40", "41", "42", "43", "44", "45", "46", "47", "48", 
                  "Mean")

# function to extract total N surplus at NUTS level
function_extract_grid_to_shapefile <- function(method) {
  NUTS = "0"
  shp <- st_read(paste0("Project_NUTS_", NUTS, ".shp"))
  NUTS_ID_df <- read.csv("NUTS_", NUTS, "_ID.csv")
  
  # NetCDF file for P surplus method 
  fname <- paste0("dataset/P_sur_total_kg_ha_grid/P_sur_total_kg_ha_grid_area_1850-2019_method_", method, "_v1.0.nc") 
  
  area <- raster("general/Hyde/Eu_garea_hyde_ha.tif") %>% # area of grid cell in ha
    crop(., c(-11, 42, 35.5, 72)) # assign croped extent of the files
  
  # create an empty datalist
  datalist = list()
  
  #Loop over years to extract P surplus by summing gridded data at NUTS level from 1850-2019 #####
  yStart = 1850
  yEnd = 2019
  
  for (y in yStart:yEnd) {
    yy =  y - yStart + 1
    r_kg_ha <- raster(fname , varname = "Psurplus", band = yy)
    r_kg <- r_kg_ha  * area # convert P surplus into kg
    e <- exactextractr::exact_extract(r_kg, shp, 'sum') # extract gridded data over NUTS shape files using by summing them
    datalist[[yy]] <- as.data.frame(e) 
  }
  
  # join dataframes
  Total_df = do.call(cbind, datalist) # Bind all years together 
  Total_df$NUTS_ID <- shp$NUTS_ID     # assign NUTS ID to the dataframe
  colnames(Total_df) <- c(yStart:yEnd, "NUTS_ID") # define column names 
  
  # arrange dataframe 
  P_sur_total_kg  <- Total_df  %>%
    pivot_longer(., cols = -NUTS_ID, names_to = "Year", values_to = "P_sur_kg")  %>%
    inner_join(., NUTS_ID_df, by = c("NUTS_ID"))  %>% # Join with the River ID's 
    mutate(Year = as.numeric(Year))
  
  Area <- read_xls(paste0("Area_ha_NUTS_0.xls")) %>%
    select(NUTS_ID, Area_ha) # We derived area of the NUTS shapefile in ha using ArcGIS tool
  
  P_sur_total_kg_ha <- full_join(P_sur_total_kg, Area, by = c ("NUTS_ID")) %>%
    group_by(NUTS_ID, Year)%>%
    mutate(P_sur_total_kg_ha = P_sur_kg/Area_ha)         # estimate vales in kg/ha of physical area for NUTS
  
  # assign name to the output file of N surplus (kg/ha of physical area/ha)
  fname = paste0("P_sur_total_kg_ha_physical_area_1850-2019_method_", method, ".csv")
  # write file for the total N surplus in csv format
  write.csv(P_sur_total_kg_ha, fname)
  
}
# apply the function over the created list
lapply(methods_list, function_extract_grid_to_shapefile)


###########################################################################################################################################################  
#2. derive mean and standard deviation of 48 N surplus estimate at NUTS level
###########################################################################################################################################################  

# Reading and selecting the relevant columns for each method
read_and_select <- function(i) {
  fname <- sprintf("dataset/P_sur_total_kg_ha_physical_area_1850-2019_method_%d.csv", i)
  col_name <- sprintf("P_sur_total_kg_ha_%d", i)
  read.csv(fname) %>%
    select(NUTS_ID, Year, !!col_name := P_sur_total_kg_ha)
}

# Creating a list to store the dataframes
data_list <- lapply(1:48, read_and_select)

# Joining all dataframes together
df <- Reduce(function(x, y) left_join(x, y, by = c("NUTS_ID", "Year")), data_list)

# Calculating the mean and standard deviation
df_summary <- df %>%
  group_by(NUTS_ID, Year) %>%
  mutate(
    P_sur_kg_ha_mean = rowMeans(select(., starts_with("P_sur_total_kg_ha_")), na.rm = TRUE),
    P_sur_kg_ha_sd = apply(select(., starts_with("P_sur_total_kg_ha_")), 1, sd, na.rm = TRUE)
  ) %>%
  ungroup() %>%
  select(NUTS_ID, Year, P_sur_kg_ha_mean, P_sur_kg_ha_sd) %>%
  as.data.frame()

# Writing the output to a CSV file
write_csv(df_summary, "P_sur_total_kg_ha_NUTS0_v1.csv")

